import os
from conans import ConanFile, CMake, tools


class OpenCLHeadersTestConan(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    generators = "cmake"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()

    def test(self):
        if not tools.cross_building(self.settings):
            os.chdir("bin")
            self.run(".%sexample" % os.sep)



from cpt.packager import ConanMultiPackager
import datetime

if __name__ == "__main__":
    builder = ConanMultiPackager(
    	username="khronos",
    	login_username="khronos",
    	channel="stable"
	)
    builder.add_common_builds()
    builder.run()



# Copyright (c) 2021 The Khronos Group Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict
from collections import namedtuple

import argparse
import sys
import urllib
import xml.etree.ElementTree as etree
import urllib.request

# parse_xml - Helper function to parse the XML file from a URL or local file.
def parse_xml(path):
    file = urllib.request.urlopen(path) if path.startswith("http") else open(path, 'r')
    with file:
        tree = etree.parse(file)
        return tree

# noneStr - returns string argument, or "" if argument is None.
def noneStr(s):
    if s:
        return s
    return ""

def parse_args():
    parser = argparse.ArgumentParser()

    # To pull the latest registry file from GitHub, pass:
    # -registry "https://raw.githubusercontent.com/KhronosGroup/OpenCL-Registry/master/xml/cl.xml"

    parser.add_argument('-registry', action='store',
                        default='cl.xml',
                        help='Use specified registry file instead of cl.xml')
    parser.add_argument('-o', action='store', dest='directory',
                        default='.',
                        help='Create target and related files in specified directory')

    args = parser.parse_args()
    return args

def load_spec(args):
    specpath = args.registry

    print('Parsing XML file from: ' + specpath)
    spec = parse_xml(specpath)
    return spec

def get_apitypedefs(spec):
    typedefs = OrderedDict()
    Typedef = namedtuple('Typedef', 'Typedef Name')
    print('Generating API typedef dictionary...')
    for type in spec.findall('types/type'):
        if type.get('category') == 'define':
            if noneStr(type.text).startswith("typedef"):
                typedef = noneStr(type.text)
                name = ""
                for elem in type:
                    if elem.tag == 'name':
                        name = noneStr(elem.text)
                    else:
                        typedef = typedef + noneStr(elem.text) + noneStr(elem.tail)
                typedef = typedef.strip()
                name = name.strip()
                typedefs[name] = Typedef(typedef, name)
    return typedefs

def get_apimacros(spec):
    macros = OrderedDict()
    Macro = namedtuple('Macro', 'Define Name Macro')
    print('Generating API macro dictionary...')
    for type in spec.findall('types/type'):
        if type.get('category') == 'define':
            if noneStr(type.text).startswith("#define"):
                define = noneStr(type.text)
                name = ""
                macro = ""
                for elem in type:
                    if elem.tag == 'name':
                        name = noneStr(elem.text)
                        macro = macro + noneStr(elem.tail)
                define = define.strip()
                name = name.strip()
                macro = macro.rstrip() # keep spaces on the left!
                macros[name] = Macro(define, name, macro)
    return macros

def get_apistructs(spec):
    structs = OrderedDict()
    Struct = namedtuple('Struct', 'Name Members')
    StructMember = namedtuple('StructMember', 'Type TypeEnd Name')
    print('Generating API struct dictionary...')
    for type in spec.findall('types/type'):
        if type.get('category') == 'struct':
            name = type.get('name')
            mlist = []
            for member in type.findall('member'):
                mtype = noneStr(member.text)
                mtypeend = ""
                mname = ""
                for elem in member:
                    if elem.tag == 'name':
                        mname = noneStr(elem.text)
                        mtypeend = noneStr(elem.tail)
                    elif elem.tag == 'enum':
                        # Assumes any additional enums are for array limits, e.g.:
                        # <member><type>char</type><name>name</name>[<enum>CL_NAME_VERSION_MAX_NAME_SIZE</enum>]</member>
                        mtypeend = mtypeend + noneStr(elem.text) + noneStr(elem.tail)
                    else:
                        mtype = mtype + noneStr(elem.text) + noneStr(elem.tail)
                mtype = mtype.strip()
                mtypeend = mtypeend.strip()
                mname = mname.strip()
                mlist.append(StructMember(mtype, mtypeend, mname))
            structs[name] = Struct(name, mlist)
    return structs

def get_apienums(spec):
    enums = OrderedDict()
    Enum = namedtuple('Enum', 'Value Bitpos Name')
    print('Generating API enum dictionary...')
    for enum in spec.findall('enums/enum'):
        value = enum.get('value')
        bitpos = enum.get('bitpos')
        name = enum.get('name')
        enums[name] = Enum(value, bitpos, name)
    return enums

def get_apisigs(spec):
    apisigs = OrderedDict()
    ApiSignature = namedtuple('ApiSignature', 'Name RetType Params Suffix')
    ApiParam = namedtuple('ApiParam', 'Type TypeEnd Name')
    print('Generating API signatures dictionary...')
    for command in spec.findall('commands/command'):
        suffix = noneStr(command.get('suffix'))
        proto = command.find('proto')
        ret = noneStr(proto.text)
        name = ""
        params = ""
        for elem in proto:
            if elem.tag == 'name':
                name = noneStr(elem.text) + noneStr(elem.tail)
            else:
                ret = ret + noneStr(elem.text) + noneStr(elem.tail)
        ret = ret.strip()
        name = name.strip()

        plist = []
        for param in command.findall('param'):
            ptype = noneStr(param.text)
            ptypeend = ""
            pname = ""
            for elem in param:
                if elem.tag == 'name':
                    pname = noneStr(elem.text)
                    ptypeend = noneStr(elem.tail)
                else:
                    ptype = ptype + noneStr(elem.text) + noneStr(elem.tail)
            ptype = ptype.strip()
            ptypeend = ptypeend.strip()
            pname = pname.strip()
            plist.append(ApiParam(ptype, ptypeend, pname))

        # For an empty parameter list (for e.g. clUnloadCompiler), add a single
        # unnamed void parameter to make generation easier.
        if len(plist) == 0:
            plist.append(ApiParam("void", "", ""))

        apisigs[name] = ApiSignature(name, ret, plist, suffix)
    return apisigs

def get_coreapis(spec, apisigs):
    coreapis = OrderedDict()
    print('Generating core API dictionary...')
    for feature in spec.findall('feature'):
        version = noneStr(feature.get('name'))

        alist = []
        for function in feature.findall('require/command'):
            name = function.get('name')
            alist.append(apisigs[name])
        coreapis[version] = alist
    return coreapis

def get_extapis(spec, apisigs):
    extapis = OrderedDict()
    print('Generating API extensions dictionary...')
    for feature in spec.findall('extensions/extension'):
        extension = noneStr(feature.get('name'))

        alist = []
        for function in feature.findall('require/command'):
            name = function.get('name')
            alist.append(apisigs[name])
        extapis[extension] = alist
    return extapis

def get_apis(spec, apisigs):
    return (get_coreapis(spec, apisigs), get_extapis(spec, apisigs))



#!/usr/bin/python3

# Copyright (c) 2021 The Khronos Group Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import gen
import re

from mako.template import Template
from mako.exceptions import RichTraceback

from collections import OrderedDict
from collections import namedtuple

def getDisableWarningIncludeString(include):
    warningPush = """#if defined(_MSC_VER)
#if _MSC_VER >=1500
#pragma warning( push )
#pragma warning( disable : 4201 )
#pragma warning( disable : 5105 )
#endif
#endif
"""
    warningPop = """
#if defined(_MSC_VER)
#if _MSC_VER >=1500
#pragma warning( pop )
#endif
#endif"""
    return warningPush + include + warningPop

def getWin32OnlyIncludeString(include):
    return """#if defined(_WIN32)
""" + include + """
#endif"""

if __name__ == "__main__":
    args = gen.parse_args()
    spec = gen.load_spec(args)

    typedefs = gen.get_apitypedefs(spec)
    macros = gen.get_apimacros(spec)
    structs = gen.get_apistructs(spec)
    enums = gen.get_apienums(spec)
    apisigs = gen.get_apisigs(spec)

    coreapis = gen.get_coreapis(spec, apisigs)
    extapis = gen.get_extapis(spec, apisigs)

    try:
        cl_ext_h_template = Template(filename='cl_ext.h.mako', input_encoding='utf-8')
        cl_function_types_h_template = Template(filename='cl_function_types.h.mako', input_encoding='utf-8')

        print('Generating cl_dx9_media_sharing.h...')
        text = cl_ext_h_template.render(
            genExtensions={
                'cl_khr_dx9_media_sharing',
                'cl_intel_dx9_media_sharing',
                'cl_intel_sharing_format_query_dx9',
            },
            guard="OPENCL_CL_DX9_MEDIA_SHARING_H_",
            includes=getWin32OnlyIncludeString(getDisableWarningIncludeString("#include <d3d9.h>")),
            generate_pfn_typedefs=False,
            spec=spec,
            typedefs=typedefs,
            macros=macros,
            structs=structs,
            enums=enums,
            apisigs=apisigs,
            coreapis=coreapis,
            extapis=extapis)
        text = re.sub(r'\r\n', r'\n', text)
        with open(args.directory + '/cl_dx9_media_sharing.h', 'w') as gen:
            gen.write(text)

        print('Generating cl_d3d10.h...')
        text = cl_ext_h_template.render(
            genExtensions={
                'cl_khr_d3d10_sharing',
                'cl_intel_sharing_format_query_d3d10',
            },
            guard="OPENCL_CL_D3D10_H_",
            includes=getDisableWarningIncludeString("#include <d3d10.h>"),
            generate_pfn_typedefs=False,
            spec=spec,
            typedefs=typedefs,
            macros=macros,
            structs=structs,
            enums=enums,
            apisigs=apisigs,
            coreapis=coreapis,
            extapis=extapis)
        text = re.sub(r'\r\n', r'\n', text)
        with open(args.directory + '/cl_d3d10.h', 'w') as gen:
            gen.write(text)

        print('Generating cl_d3d11.h...')
        text = cl_ext_h_template.render(
            genExtensions={
                'cl_khr_d3d11_sharing',
                'cl_intel_sharing_format_query_d3d11'
            },
            guard="OPENCL_CL_D3D11_H_",
            includes=getDisableWarningIncludeString("#include <d3d11.h>"),
            generate_pfn_typedefs=False,
            spec=spec,
            typedefs=typedefs,
            macros=macros,
            structs=structs,
            enums=enums,
            apisigs=apisigs,
            coreapis=coreapis,
            extapis=extapis)
        text = re.sub(r'\r\n', r'\n', text)
        with open(args.directory + '/cl_d3d11.h', 'w') as gen:
            gen.write(text)

        print('Generating cl_egl.h...')
        text = cl_ext_h_template.render(
            genExtensions={'cl_khr_egl_event', 'cl_khr_egl_image'},
            guard="OPENCL_CL_EGL_H_",
            generate_pfn_typedefs=False,
            spec=spec,
            typedefs=typedefs,
            macros=macros,
            structs=structs,
            enums=enums,
            apisigs=apisigs,
            coreapis=coreapis,
            extapis=extapis)
        text = re.sub(r'\r\n', r'\n', text)
        with open(args.directory + '/cl_egl.h', 'w') as gen:
            gen.write(text)

        print('Generating cl_gl.h...')
        text = cl_ext_h_template.render(
            genExtensions={
                'cl_khr_gl_depth_images',
                'cl_khr_gl_event',
                'cl_khr_gl_msaa_sharing',
                'cl_khr_gl_sharing',
                'cl_intel_sharing_format_query_gl',
            },
            guard="OPENCL_CL_GL_H_",
            generate_pfn_typedefs=False,
            spec=spec,
            typedefs=typedefs,
            macros=macros,
            structs=structs,
            enums=enums,
            apisigs=apisigs,
            coreapis=coreapis,
            extapis=extapis)
        text = re.sub(r'\r\n', r'\n', text)
        with open(args.directory + '/cl_gl.h', 'w') as gen:
            gen.write(text)

        print('Generating cl_layer.h...')
        text = cl_ext_h_template.render(
            genExtensions={'cl_loader_layers'},
            guard="OPENCL_CL_LAYER_H_",
            includes='#include <CL/cl_icd.h>',
            generate_pfn_typedefs=True,
            spec=spec,
            typedefs=typedefs,
            macros=macros,
            structs=structs,
            enums=enums,
            apisigs=apisigs,
            coreapis=coreapis,
            extapis=extapis)
        text = re.sub(r'\r\n', r'\n', text)
        with open(args.directory + '/cl_layer.h', 'w') as gen:
            gen.write(text)

        print('Generating cl_va_api_media_sharing_intel.h...')
        text = cl_ext_h_template.render(
            genExtensions={
                'cl_intel_va_api_media_sharing',
                'cl_intel_sharing_format_query_va_api',
            },
            guard="OPENCL_CL_VA_API_MEDIA_SHARING_INTEL_H_",
            includes='#include <va/va.h>',
            generate_pfn_typedefs=False,
            spec=spec,
            typedefs=typedefs,
            macros=macros,
            structs=structs,
            enums=enums,
            apisigs=apisigs,
            coreapis=coreapis,
            extapis=extapis)
        text = re.sub(r'\r\n', r'\n', text)
        with open(args.directory + '/cl_va_api_media_sharing_intel.h', 'w') as gen:
            gen.write(text)

        print('Generating cl_ext.h...')
        text = cl_ext_h_template.render(
            genExtensions={},
            guard="OPENCL_CL_EXT_H_",
            generate_pfn_typedefs=False,
            spec=spec,
            typedefs=typedefs,
            macros=macros,
            structs=structs,
            enums=enums,
            apisigs=apisigs,
            coreapis=coreapis,
            extapis=extapis)
        text = re.sub(r'\r\n', r'\n', text)
        with open(args.directory + '/cl_ext.h', 'w') as gen:
            gen.write(text)

        print('Generating cl_function_types.h...')
        text = cl_function_types_h_template.render(
            spec=spec,
            apisigs=apisigs,
            coreapis=coreapis,
            extapis=extapis)
        text = re.sub(r'\r\n', r'\n', text)
        with open(args.directory + '/cl_function_types.h', 'w') as gen:
            gen.write(text)

    except:
        traceback = RichTraceback()
        for (filename, lineno, function, line) in traceback.traceback:
            print('%s(%s) : error in %s' % (filename, lineno, function))
            print('    ', line)
        print('%s: %s' % (str(traceback.error.__class__.__name__), traceback.error))



from conans import ConanFile, tools, CMake
import os


class OpenCLHeadersConan(ConanFile):
    name = "opencl-headers"
    version = "20190502"
    license = "Apache-2.0"
    author = "Khronos Group <webmaster@khronos.org>"
    url = "https://github.com/KhronosGroup/OpenCL-ICD-Loader"
    description = "Khronos OpenCL Headers"
    topics = ("khronos", "opencl", "headers")
    exports_sources = "CMakeLists.txt", "CL/*"
    no_copy_source = True

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()
        cmake.install()

    def package_id(self):
        self.info.header_only()